function facpack
% The toolbox FACPACK provides software for the following problems in
% multivariate curve resolution:
% 1. baseline correction,
% 2. computation of 2- and 3-component factorizations and the area of feasible
%    solutions,
% 3. computation of 4-component factorizations and the area of feasible
%    solutions,
% 4. generalized Borgen plots for the computation of the AFS for data 
%    including slightly negative entries
% 5. combination of complementarity and AFS for 3-component systems
% 6. combination of complementarity and AFS for 4-component systems
% 7. peak group analysis for the extraction of pure component spectra
% 8. FACPACK library to view, export, add or delete pure component spectra
%    or concentration profiles
% Copyright (c) 2013 Mathias Sawall, Annekathrin Juerss, University of Rostock, Department of Mathematics.
% Copyright (c) 2023 Mathias Sawall, Annekathrin Juerss, Tomass Andersons, Martina Beese, Lukas Prestin, University of Rostock, Department of Mathematics.
% 
% The software can be used for academic, research and other similar noncommercial uses. The user
% acknowledges that this software is still in the development stage and that it is provided by the
% copyright holders and contributors "as is" and any express or implied warranties, fitness for a
% particular purpose are disclaimed. In no event shall the copyright owner or contributors be liable
% for any direct, indirect, incidental, special, exemplary, or consequential damages.
% 
% The copyright holders provide no reassurances that the source code provided does not infringe any
% patent, copyright, or any other intellectual property rights of third parties. The copyright
% holders disclaim any liability to any recipient for claims brought against recipient by any third
% party for infringement of that parties intellectual property rights.
% 
% The user has to respect the lapack license, see http://www.netlib.org/lapack/LICENSE.txt, and
% the license for the ACM routine nl2sol, see http://www.acm.org/publications/policies/softwarecrnotice,
% which are used in the associated C-code. Furthermore the user has to respect the license for the 
% file Mouse3D.m from the Mathworks File Exchange, see 
% http://www.mathworks.com/matlabcentral/fileexchange/view_license?file_info_id=18266

version = 1.3;
fname = mfilename;
calcf = mfilename('fullpath');
calcf = calcf(1:end-length(fname));
calcfp = [calcf, 'subfcnsFP'];
path(path, calcf);
path(path, calcfp);

fig = figure(200);
pause(.1);

scrsz = get(0,'ScreenSize');

set(fig,'Position',[scrsz(3)/4 scrsz(4)/3 scrsz(3)/1.5 scrsz(4)/3]);

pause(.5);
figpos = get(fig,'Position');

set(fig,'Name',['FACPACK (version ',num2str(version, '%2.2f'),'-alpha)'],'NumberTitle','off','MenuBar','none','Toolbar','none','color','white');

if ispc
    basicfsz = 5;
    basichi = 0.06;
else
    basicfsz = 5;
    basichi = 0.05;
end

Styles = {'bitstream charter';'clean';'Courier';'courier 10 pitch';'frank ruehl';'Helvetica';'Helvetica-Narrow';'itc bookman';'urw bookman l'};
sty = Styles{6};
bgc = 0.95*ones(3,1);

nb = 1;

na = 4;
ha = 0.8/na;
fa = 1.1;
hz = 0.9;
dis = 0.01;
ncol = 4;
wd0 = 0.135;
wd = (1-wd0-(ncol+2)*dis)/ncol;

nc = 0;
Catergory(1) = uicontrol('Style','text','String','Basic Modules',...
    'HorizontalAlignment','center','Fontsize',2.8*basicfsz,'BackgroundColor','w',...
    'Units','normalized','FontName',sty,'FontWeight','bold','Position',[(nc+1)*dis+nc*wd, hz-fa*nb*ha-0.03, wd0, ha]);

Catergory(1) = uicontrol('Style','text','String','Expert Modules',...
    'HorizontalAlignment','center','Fontsize',2.8*basicfsz,'BackgroundColor','w',...
    'Units','normalized','FontName',sty,'FontWeight','bold','Position',[(nc+1)*dis+nc*wd, hz-fa*(nb+1)*ha-0.07, wd0, ha]);


Catergory(2) = uicontrol('Style','text','String','Peak Group Analysis',...
    'HorizontalAlignment','center','Fontsize',2.4*basicfsz,'BackgroundColor','w',...
    'Units','normalized','FontName',sty,'FontWeight','bold','Position',[(nc+2)*dis+nc*wd+wd0, 0.91, wd, 0.06]);

Button(nb) = uicontrol('Style','pushbutton','String','<html><center>Peak Group Analysis<br/>(for series of IR and Raman spectra)<html><center>','HorizontalAlignment','left','Fontsize',2*basicfsz,'FontWeight','bold',...
    'Units','normalized','FontName',sty,'Position',[(nc+2)*dis+nc*wd+wd0, hz-fa*nb*ha+0.02, wd, ha],'Callback',{@buttonPGA, version});
nb = nb+1;

Button(nb) = uicontrol('Style','pushbutton','String','Pure Component Decomposition','HorizontalAlignment','left','Fontsize',2*basicfsz,'FontWeight','bold',...
    'Units','normalized','FontName',sty,'Position',[(nc+2)*dis+nc*wd+wd0, hz-fa*nb*ha, wd, ha],'Enable','off','Callback',{@buttonPCD, version});
nb = nb+1;

Button(nb) = uicontrol('Style','pushbutton','String','<html><center>Individual FACPACK library<br/>(user saved profiles)<html><center>','HorizontalAlignment','left','Fontsize',2*basicfsz,'FontWeight','bold',...
    'Units','normalized','FontName',sty,'Position',[(nc+2)*dis+nc*wd+wd0, hz-fa*nb*ha, wd, ha],'Callback',{@buttonLIB, version});
nb = nb+1;
nc = nc+1;


Catergory(3) = uicontrol('Style','text','String','Area of Feasible Solutions',...
    'HorizontalAlignment','center','Fontsize',2.4*basicfsz,'BackgroundColor','w',...
    'Units','normalized','FontName',sty,'FontWeight','bold','Position',[(nc+2)*dis+nc*wd+wd0, 0.91, wd, 0.06]);
Button(nb) = uicontrol('Style','pushbutton','String','<html><center>Polygon Inflation<br/>(2 & 3 components)</center></html>','HorizontalAlignment','left','Fontsize',2*basicfsz,'FontWeight','bold',...
    'Units','normalized','FontName',sty,'Position',[(nc+2)*dis+nc*wd+wd0, hz-fa*(nb-3)*ha+0.02, wd, ha],'Callback',{@buttonAFS, version});
nb = nb+1;

Button(nb) = uicontrol('Style','pushbutton','String','<html><center>Ray Casting<br/>(4 components)<html><center>','HorizontalAlignment','left','Fontsize',2*basicfsz,'FontWeight','bold',...
    'Units','normalized','FontName',sty,'Position',[(nc+2)*dis+nc*wd+wd0, hz-fa*(nb-3)*ha, wd, ha],'Callback',{@buttonRAY, version});
nb = nb+1;

Button(nb) = uicontrol('Style','pushbutton','String','<html><center>Generalized Borgen Plots<br/>(3 components)</center></html>','HorizontalAlignment','left','Fontsize',2*basicfsz,'FontWeight','bold',...
    'Units','normalized','FontName',sty,'Position',[(nc+2)*dis+nc*wd+wd0, hz-fa*(nb-3)*ha, wd, ha],'Callback',{@buttonBKP, version});
nb = nb+1;
nc = nc+1;


Catergory(4) = uicontrol('Style','text','String','Duality & Area of Feasibe solutions',...
    'HorizontalAlignment','center','Fontsize',2.4*basicfsz,'BackgroundColor','w',...
    'Units','normalized','FontName',sty,'FontWeight','bold','Position',[(nc+2)*dis+nc*wd+wd0, 0.91, wd, 0.06]);
Button(nb) = uicontrol('Style','pushbutton','String','<html><center>Duality & AFS<br/>(3 components)<html><center>','HorizontalAlignment','left','Fontsize',2*basicfsz,'FontWeight','bold',...
    'Units','normalized','FontName',sty,'Position',[(nc+2)*dis+nc*wd+wd0, hz-fa*(nb-6)*ha+0.02, wd, ha],'Callback',{@buttonCAA, version});
nb = nb+1;

Button(nb) = uicontrol('Style','pushbutton','String','<html><center>Duality & AFS<br/>(4 components)<html><center>','HorizontalAlignment','left','Fontsize',2*basicfsz,'FontWeight','bold',...
    'Units','normalized','FontName',sty,'Position',[(nc+2)*dis+nc*wd+wd0, hz-fa*(nb-6)*ha, wd, ha],'Callback',{@buttonCA4, version});
nb = nb+1;
nc = nc+1;


Catergory(1) = uicontrol('Style','text','String','Data Preparation and Analysis',...
    'HorizontalAlignment','center','Fontsize',2.4*basicfsz,'BackgroundColor','w',...
    'Units','normalized','FontName',sty,'FontWeight','bold','Position',[(nc+2)*dis+nc*wd+wd0, 0.91, wd, 0.06]);

Button(nb) = uicontrol('Style','pushbutton','String','Baseline Correction','HorizontalAlignment','left','Fontsize',2*basicfsz,'FontWeight','bold',...
    'Units','normalized','FontName',sty,'Position',[(nc+2)*dis+nc*wd+wd0,  hz-fa*(nb-8)*ha+0.02, wd, ha],'Callback',{@buttonBLA, version});
nb = nb+1;

Button(nb) = uicontrol('Style','pushbutton','String','<html><center>Essential Spectral Information<br/>(also for noisy data)</center></html>',...
    'HorizontalAlignment','left','Fontsize',2*basicfsz,'FontWeight','bold',...
    'Units','normalized','FontName',sty,'Position',[(nc+2)*dis+nc*wd+wd0, hz-fa*(nb-8)*ha, wd, ha],'Callback',{@buttonEDP, version});
nb = nb+1;

Button(nb) = uicontrol('Style','pushbutton','String','<html><center>Inner & Outer Polytopes<br/>(also for noisy data)</center></html>','HorizontalAlignment','left','Fontsize',2*basicfsz,'FontWeight','bold',...
    'Units','normalized','FontName',sty,'Position',[(nc+2)*dis+nc*wd+wd0, hz-fa*(nb-8)*ha, wd, ha],'Callback',{@buttonIPFP, version});
nb = nb+1;

Button(nb) = uicontrol('Style','pushbutton','String','<html><center>Construction & Modification of Data<br/>using inner & outer polygons</center></html>',...
    'HorizontalAlignment','left','Fontsize',2*basicfsz,'FontWeight','bold','Enable','off',...
    'Units','normalized','FontName',sty,'Position',[(nc+2)*dis+nc*wd+wd0, hz-fa*(nb-8)*ha, wd, ha],'Callback',{@buttonIP4D, version});
nb = nb+1;
nc = nc+1;


function buttonBLA(hObject, eventdata, version)
figure(201);
subGUI4BLA(version,201);

function buttonPGA(hObject, eventdata, version)
figure(202);
subGUI4PGA(version,202);

function buttonPCD(hObject, eventdata, version)
figure(203);
subGUI4PCD(version,203)

function buttonAFS(hObject, eventdata, version)
figure(204);
subGUI4AFS(version,204);

function buttonRAY(hObject, eventdata, version)
figure(205);
subGUI4RAY(version,205);

function buttonBKP(hObject, eventdata, version)
figure(206);
subGUI4BKP(version,206);

function buttonCAA(hObject, eventdata, version)
figure(207);
subGUI4CAA(version,207);

function buttonCA4(hObject, eventdata, version)
figure(208);
subGUI4CA4(version,208);

function buttonLIB(hObject, eventdata, version)
figure(209);
subGUI4LIB(version,209);

function buttonEDP(hObject, eventdata, version)
figure(210);
subGUI4EDP(version,210);

function buttonIP4D(hObject, eventdata, version)
figure(211);
subGUI4IP4D(version,211);

function buttonIPFP(hObject, eventdata, version)
figure(212);
subGUI4IFP(version,212);
